using System;
using System.Collections.Generic;
using System.Configuration;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Transactions;
using System.Text;

namespace Compensation
{


    [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerSession, ConcurrencyMode = ConcurrencyMode.Single, ReleaseServiceInstanceOnTransactionComplete = true)]
    public class MyService: IMyServiceContract
    {
        private class MyExtension : IExtension<InstanceContext> 
        {
            private List<string> messages = null;
            public bool? fail = null;


            public void Attach(InstanceContext owner)
            {
                this.messages = new List<string>();
                this.fail = (!(bool.Parse(ConfigurationManager.AppSettings["Succeed"])));
            }

            public void Detach(InstanceContext owner)
            {
                this.messages = null;
            }

            public List<string> Messages
            {
                get
                {
                    return this.messages;
                }
            }

            public bool? Fail
            {
                get
                {
                    return this.fail;
                }
            }
            
        }

        [OperationBehavior(TransactionScopeRequired = true, TransactionAutoComplete = false)]
        public void FirstOperation(string input)
        {
            Console.WriteLine("Read {0}.", input);

            MyExtension extension = new MyExtension();
            OperationContext.Current.InstanceContext.Extensions.Add(extension);
            extension.Messages.Add(input);
            

            try
            {
                MyClient targetClient = new MyClient("TargetService");
                targetClient.Open();
                targetClient.FirstOperation(input);
                targetClient.Close();
                
            }
            catch (Exception exception)
            {
                this.Compensate(extension);

                throw exception;
            }
             
            
        }

        [OperationBehavior(TransactionScopeRequired = true, TransactionAutoComplete = true)]
        public void SecondOperation(string input)
        {
            Console.WriteLine("Read {0}.", input);

            MyExtension extension = OperationContext.Current.InstanceContext.Extensions.Find<MyExtension>();
            if (extension != null)
            {
                extension.Messages.Add(input);
            }
            
            try
            {
                if ((extension != null) && (extension.Fail != null) && (extension.Fail.Value))
                {
                    throw new Exception("Something bad.");
                }

                MyClient targetClient = new MyClient("TargetService");
                targetClient.Open();
                targetClient.SecondOperation(input);
                targetClient.Close();
                
                
            }
            catch (Exception exception)
            {
                this.Compensate(extension);

                throw exception;
            }
            
        }

        private void Compensate(MyExtension extension)
        {
            using (TransactionScope transaction = new TransactionScope(TransactionScopeOption.RequiresNew))
            {
                StringBuilder buffer = null;
                if (extension != null)
                {

                    foreach (string message in extension.Messages)
                    {
                        if (buffer == null)
                        {
                            buffer = new StringBuilder();
                            buffer.Append(message);
                        }
                        else
                        {
                            buffer.Append(";");
                            buffer.Append(message);
                        }
                    }
                }

                CompensationClient compensationClient = new CompensationClient("CompensationService");
                compensationClient.Open();


                compensationClient.Compensate(Message.CreateMessage(MessageVersion.Soap12WSAddressing10, "*", buffer == null ? string.Empty : buffer.ToString()));
                compensationClient.Close();
                transaction.Complete();
            }
        }



        

        
    }
}
