using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design.Serialization;
using System.Configuration;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Configuration;
using System.Text;
using System.Threading;
using System.Xml;

namespace Extensibility
{
    [ServiceContract]
    public interface IBridge
    {
        [OperationContract]
        int GetDropRate();
        [OperationContract]
        void ReportReceivedCount();
        [OperationContract]
        void ReportDroppedCount();
        [OperationContract]
        void ReportForwardedCount();
    }

    public class MessageInterceptor : IMessageInterceptor
    {
        private Random randomizer = null;
        private IBridge proxy = null;
        private int dropRate = 0;

        public MessageInterceptor()
        {
            this.randomizer = new Random(DateTime.Now.Millisecond);

        }

        public void ProcessSend(ref Message message)
        {
            if (this.proxy == null)
            {
                this.proxy = new ChannelFactory<IBridge>("Bridge").CreateChannel();
            }
            string action = message.Headers.Action;
            Console.WriteLine(string.Format("Voting on message {0} ...", action));
            if (!(action.Contains("IImageTransfer")))
            {
                return;
            }
            this.proxy.ReportReceivedCount();
            int randomNumber = this.randomizer.Next(100);
            this.dropRate = proxy.GetDropRate();
            if ((this.dropRate != 0) && (randomNumber <= this.dropRate))
            {
                Console.WriteLine("Dropping message.");
                message = null;
                this.proxy.ReportDroppedCount();
                return;
            }
            Console.WriteLine("Forwarding message.");
            this.proxy.ReportForwardedCount();

        }

        public void ProcessReceive(ref Message message)
        {

        }
    }


    public interface IMessageInterceptor
    {
        void ProcessReceive(ref Message message);
        void ProcessSend(ref Message message);
    }

}
