using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;

namespace Extensibility
{
	public class MyCustomChannelFactory<TChannel> : ChannelFactoryBase<TChannel>, ICustomChannelManager
	{
		IChannelFactory<TChannel> innerChannelFactory;
		
		public MyCustomChannelFactory()
		{
		}

		public override MessageVersion MessageVersion
		{
			get
			{
				return this.InnerChannelFactory.MessageVersion;
			}
		}

		public override string Scheme
		{
			get
			{
				return this.InnerChannelFactory.Scheme;
			}
		}

		public IChannelFactory<TChannel> InnerChannelFactory
		{
			get { return this.innerChannelFactory; }
			set { this.innerChannelFactory = value; }
		}

		protected override void OnOpen(TimeSpan timeout)
		{
			this.innerChannelFactory.Open(timeout);
		}

		protected override IAsyncResult OnBeginOpen(TimeSpan timeout, AsyncCallback callback, object state)
		{
			return this.innerChannelFactory.BeginOpen(timeout, callback, state);
		}

		protected override void OnEndOpen(IAsyncResult result)
		{
			this.innerChannelFactory.EndOpen(result);
		}

		protected override void OnClose(TimeSpan timeout)
		{
			this.innerChannelFactory.Close(timeout);
		}

		protected override IAsyncResult OnBeginClose(TimeSpan timeout, AsyncCallback callback, object state)
		{
			return this.innerChannelFactory.BeginClose(timeout, callback, state);
		}

		protected override void OnEndClose(IAsyncResult result)
		{
			this.innerChannelFactory.EndClose(result);
		}

		protected override TChannel OnCreateChannel(EndpointAddress to, Uri via)
		{
			TChannel innerChannel = this.InnerChannelFactory.CreateChannel(to, via);
			if (typeof(TChannel) == typeof(IOutputChannel))
			{
				return (TChannel)(object)new MyCustomOutputChannel(this, (IOutputChannel)innerChannel);
			}
			
			throw new InvalidOperationException();
		}

		void ICustomChannelManager.OnReceive(Message message)
		{
			
		}

		void ICustomChannelManager.OnSend(Message message)
		{
			Console.WriteLine("Here I am playing with outgoing messages in an output channel.");
		}

		void ThrowIfInnerFactoryNotSet()
		{
			if (this.InnerChannelFactory == null)
			{
				throw new InvalidOperationException("Inner channel factory is not set.");
			}
		}

		

		class StatisticsRequestChannel : CustomChannelBase<IRequestChannel>, IRequestChannel
		{
			public StatisticsRequestChannel(ChannelManagerBase factory, IRequestChannel innerChannel)
				: base(factory, innerChannel)
			{
				// empty
			}

			public EndpointAddress RemoteAddress
			{
				get
				{
					return this.InnerChannel.RemoteAddress;
				}
			}

			public Uri Via
			{
				get
				{
					return this.InnerChannel.Via;
				}
			}

			public IAsyncResult BeginRequest(Message message, TimeSpan timeout, AsyncCallback callback, object state)
			{
				this.OnSend(message);
				return this.InnerChannel.BeginRequest(message, timeout, callback, state);
			}

			public IAsyncResult BeginRequest(Message message, AsyncCallback callback, object state)
			{
				this.OnSend(message);
				return this.InnerChannel.BeginRequest(message, callback, state);
			}

			public Message EndRequest(IAsyncResult result)
			{
				Message reply = this.InnerChannel.EndRequest(result);
				this.OnReceive(reply);
				return reply;
			}

			public Message Request(Message message)
			{
				this.OnSend(message);
				Message reply = this.InnerChannel.Request(message);
				this.OnReceive(reply);
				return reply;
			}

			public Message Request(Message message, TimeSpan timeout)
			{
				this.OnSend(message);
				Message reply = this.InnerChannel.Request(message, timeout);
				this.OnReceive(reply);
				return reply;
			}
		}

		

		
	}
}


