using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;

namespace Extensibility
{
	public class MyCustomChannelListener<TChannel> : ChannelListenerBase<TChannel>, ICustomChannelManager where TChannel : class, IChannel
	{
		TChannel innerChannel = null;

		internal MyCustomChannelListener()
        {
            
        }

		public new IChannelListener<TChannel> InnerChannelListener
        {
            get { return (IChannelListener<TChannel>)base.InnerChannelListener; }
            set { base.InnerChannelListener = value; }
        }

        public override Uri Uri
        {
            get
            {
                ThrowIfInnerListenerNotSet();
                return this.InnerChannelListener.Uri;
            }
        }

        public override MessageVersion MessageVersion
        {
            get
            {
                ThrowIfInnerListenerNotSet();
                return this.InnerChannelListener.MessageVersion;
            }
        }

        public override string Scheme
        {
            get
            {
                ThrowIfInnerListenerNotSet();
                return this.InnerChannelListener.Scheme;
            }
        }

        protected override void OnOpen(TimeSpan timeout)
        {
            this.InnerChannelListener.Open(timeout);
        }

        protected override IAsyncResult OnBeginOpen(TimeSpan timeout, AsyncCallback callback, object state)
        {
            return this.InnerChannelListener.BeginOpen(timeout, callback, state);
        }

        protected override void OnEndOpen(IAsyncResult result)
        {
            this.InnerChannelListener.EndOpen(result);
        }

        protected override void OnClose(TimeSpan timeout)
        {
            this.InnerChannelListener.Close(timeout);
        }

        protected override IAsyncResult OnBeginClose(TimeSpan timeout, AsyncCallback callback, object state)
        {
            return this.InnerChannelListener.BeginClose(timeout, callback, state);
        }

        protected override void OnEndClose(IAsyncResult result)
        {
            this.InnerChannelListener.EndClose(result);
        }

        

        void ThrowIfInnerListenerNotSet()
        {
            if (this.InnerChannelListener == null)
            {
                throw new InvalidOperationException("Inner listener not set.");
            }
        }

        protected override TChannel OnAcceptChannel(TimeSpan timeout)
        {
            TChannel innerChannel = this.InnerChannelListener.AcceptChannel(timeout);
            return WrapChannel(innerChannel);
        }

        protected override IAsyncResult OnBeginAcceptChannel(TimeSpan timeout, AsyncCallback callback, object state)
        {
            return this.InnerChannelListener.BeginAcceptChannel(timeout, callback, state);
        }

        protected override TChannel OnEndAcceptChannel(IAsyncResult result)
        {
            TChannel innerChannel = this.InnerChannelListener.EndAcceptChannel(result);
            return WrapChannel(innerChannel);
        }

        protected override bool OnWaitForChannel(TimeSpan timeout)
        {
            return this.InnerChannelListener.WaitForChannel(timeout);
        }

        protected override IAsyncResult OnBeginWaitForChannel(TimeSpan timeout, AsyncCallback callback, object state)
        {
            return this.InnerChannelListener.BeginWaitForChannel(timeout, callback, state);
        }

        protected override bool OnEndWaitForChannel(IAsyncResult result)
        {
            return this.InnerChannelListener.EndWaitForChannel(result);
        }

        TChannel WrapChannel(TChannel innerChannel)
        {
            if (innerChannel == null)
            {
                return null;
            }

            if (typeof(TChannel) == typeof(IInputChannel))
            {
				this.innerChannel = innerChannel;
                return (TChannel)(object)new MyCustomInputChannel(this, (IInputChannel)innerChannel);
            }
            

            // Cannot wrap this channel.
            return innerChannel;
        }



		#region ICustomChannelManager Members

		public void OnReceive(Message message)
		{
			Console.WriteLine("Here I am playing with the message in an input channel.");
			return;
		}

		public void OnSend(Message message)
		{
			throw new Exception("The method or operation is not implemented.");
		}

		#endregion

		
	}
}
