using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.ServiceModel;
using System.Text;
using System.Threading;

namespace PublicationSubscription
{
	[ServiceBehavior(InstanceContextMode=InstanceContextMode.Single,ConcurrencyMode=ConcurrencyMode.Multiple)]
	public class PublisherService: IPublisher
	{
		private RandomDataPoint randomDataPoint = null;
		private List<ISubscriber> subscribers = null;
		private object subscribersLock = null;

		public PublisherService()
		{
			this.randomDataPoint = new RandomDataPoint("Random", "Random data point", int.Parse(ConfigurationManager.AppSettings["RandomDataPointFrequencyInMilliseconds"]),false);
			this.randomDataPoint.NextValue += new NextValueHandler(this.NextValueHandler);

			this.subscribersLock = new object();
			this.subscribers = new List<ISubscriber>();

			
		}

		private void NextValueHandler(IDataPoint sender, byte[] newValue)
		{
			lock(this.subscribersLock)
			{
				for(int index = this.subscribers.Count - 1; index >= 0; index--)
				{
					try
					{
						this.subscribers[index].Notify(sender.Identifier, newValue);
					}
					catch (Exception exception)
					{
                        Console.WriteLine("Removing subscriber due to exception {0}.",exception.ToString());
						this.subscribers.RemoveAt(index);
					}
					if (this.subscribers.Count <= 0)
					{
						this.randomDataPoint.Active = false;
					}
				}
			}
		}

		

		private void ValidateDataPoint(string dataPointIdentifier, out bool valid)
		{
			valid = false;
			if (dataPointIdentifier == ((IDataPoint)this.randomDataPoint).Identifier.ToString())
			{
				valid = true;
			}
		}

	
		
		

		#region IPublisher Members

		KnownDataPoint[] IPublisher.GetKnownDataPoints()
		{
			return new KnownDataPoint[] { new KnownDataPoint(this.randomDataPoint) };
		}

		void IPublisher.Subscribe(KnownDataPoint[] dataPoints, out bool subscriptionAccepted)
		{
            Console.WriteLine("Received subscription request.");
			subscriptionAccepted = false;
			string dataPointIdentifier = null;
			if (dataPoints.Length == 1)
			{
				dataPointIdentifier = dataPoints[0].Identifier;
				this.ValidateDataPoint(dataPointIdentifier, out subscriptionAccepted);
			}

			if (subscriptionAccepted)
			{
				if (!(this.randomDataPoint.Active))
				{
					this.randomDataPoint.Active = true;
				}
				lock (this.subscribersLock)
				{
					this.subscribers.Add(OperationContext.Current.GetCallbackChannel<ISubscriber>());
				}
			}
		}

		#endregion
}
}
