using System;
using System.Collections.Generic;
using System.Configuration;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;

namespace PublicationSubscription
{
	public class Subscriber: ISubscriber, IDisposable
	{
        private class PublisherClient : DuplexClientBase<IPublisher>, IPublisher
        {
            public PublisherClient(InstanceContext callbackInstance, string endpointConfigurationName)
                : base(callbackInstance, endpointConfigurationName)
            {
            }

            #region IPublisher Members

            public KnownDataPoint[] GetKnownDataPoints()
            {
                return base.Channel.GetKnownDataPoints();
            }

            public void Subscribe(KnownDataPoint[] dataPoints, out bool subscriptionAccepted)
            {
                base.Channel.Subscribe(dataPoints, out subscriptionAccepted);
            }

            #endregion
        }

        PublisherClient publisherClient = null;

		public Subscriber()
		{
			this.publisherClient = new PublisherClient(new InstanceContext(this),"SubscriptionService");
            this.publisherClient.Open();
		}

		public void Subscribe()
		{
			KnownDataPoint[] dataPoints = this.publisherClient.GetKnownDataPoints();
			bool subscriptionAccepted = false;
			this.publisherClient.Subscribe(dataPoints, out subscriptionAccepted);
			Console.WriteLine(subscriptionAccepted ? "Subscription accepted." : "Subscription rejected.");
			
		}

		#region ISubscriber Members

		void ISubscriber.Notify(Guid dataPointIdentifier, byte[] value)
		{
			Console.WriteLine("Notified of value {0} of data point {1}.", BitConverter.ToInt32(value,0), dataPointIdentifier.ToString());
		}

		

		#endregion

		#region IDisposable Members

		void IDisposable.Dispose()
		{
			try
			{
				if (this.publisherClient != null)
				{
					if ((this.publisherClient.State == CommunicationState.Opened)&&(this.publisherClient.State != CommunicationState.Faulted))
					{
						this.publisherClient.Close();
					}
				}
			}
			catch (Exception)
			{
			}

		}

		#endregion
    }
}
