using System;
using System.Collections.Generic;
using System.Configuration;
using System.Text;

namespace PublicationSubscription
{
	public class PublishingAgent: IPublishingAgent
	{
		private string subscriberConfiguration = null;
		private RandomDataPoint randomDataPoint = null;

		public PublishingAgent(string subscriberIdentifier, string dataPointIdentifier)
		{
			this.subscriberConfiguration = subscriberIdentifier;

			this.randomDataPoint = new RandomDataPoint("Random", "Random data point", int.Parse(ConfigurationManager.AppSettings["RandomDataPointFrequencyInMilliseconds"]), false);
			if (((IDataPoint)this.randomDataPoint).Identifier.ToString() != dataPointIdentifier)
			{
				throw new ArgumentException();
			}
		}

		#region IPublishingAgent Members

		string IPublishingAgent.SubcriberConfiguration
		{
			get 
			{
				return this.subscriberConfiguration;
			}
		}

		string IPublishingAgent.DataPointIdentifier
		{
			get 
			{
				return ((IDataPoint)this.randomDataPoint).Identifier.ToString();
			}
		}

		void IPublishingAgent.Activate(object state)
		{
			this.randomDataPoint.Active = true;

			NotificationStreamWriter writer = null;

			IBufferedSubscriptionManager bufferedDataSubscriptionManager = new BufferedSubscriptionManager(this.subscriberConfiguration, 100);
			bufferedDataSubscriptionManager.AddSubscription(this.randomDataPoint);

			while (true)
			{
				using (SubscriberProxy subscriberProxy = new SubscriberProxy(this.subscriberConfiguration))
				{
					ISubscriber subscriber = (ISubscriber)subscriberProxy;
					writer = new NotificationStreamWriter(bufferedDataSubscriptionManager, long.Parse(ConfigurationManager.AppSettings["MessageCapacity"]), new TimeSpan(0, 0, 0, int.Parse(ConfigurationManager.AppSettings["UpdateFrequencyInSeconds"])), new TimeSpan(0, 0, 0, 0, int.Parse(ConfigurationManager.AppSettings["DataSourceTimeoutInMilliseconds"])));
					subscriber.Notify(writer);
					subscriberProxy.Close();
					Console.WriteLine("Batch completed.");
				}
			}
		}

		#endregion
}
}
