using System;
using System.Collections.Generic;
using System.Configuration;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Security.Cryptography;
using System.Security.Cryptography.Xml;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Security;
using System.ServiceModel.Security.Tokens;
using System.Text;

namespace WindowsCommunicationFoundationHandsOn.School
{
	
	public class QuizChild: IQuizQuestion, IQuizResponse, IQuizManagement, IDisposable
	{
		protected const string MeshIdentifierKey = @"MeshIdentifier";
		protected const string QuizQuestionEndpointKey = @"QuizQuestionEndpoint";
		protected const string QuizManagementEndpointKey = @"QuizManagementEndpoint";
		protected const string QuizResponseEndpointKey = @"QuizResponseEndpoint";
		protected const string PeerChannelAddressPrefix = @"net.p2p://";
		protected const string CertificateSubjecDistinguishedName = @"CN=FabrikamEnterprises";
        protected const string PeerBindingKey = @"PeerBinding";
        
		protected ServiceHost resolverService = null;
		protected QuizQuestionClient questionClient = null;
		protected QuizManagementClient managementClient = null;
        protected QuizResponseClient responseClient = null;
        protected string meshIdentifier = null;
		protected InstanceContext site = null;
        protected Binding binding = null;
		
		private MainForm form = null;
		private Student student = new Student();

        public QuizChild(MainForm form, string[] parameters)
        {
            this.form = form;

            this.student.Name = parameters[0];
            
            this.site = new InstanceContext(this);

            //Management
            this.managementClient = new QuizManagementClient(this.site, QuizChild.QuizManagementEndpointKey);
            this.managementClient.Open();

            //Question
            this.questionClient = new QuizQuestionClient(this.site, QuizChild.QuizQuestionEndpointKey);
            this.questionClient.Open();

            //Response
            string endpoint =
                ConfigurationManager.
                AppSettings[QuizChild.QuizResponseEndpointKey];
            EndpointAddress address =
                new EndpointAddress(
                    string.Format("{0}{1}{2}/{3}",
                        QuizChild.PeerChannelAddressPrefix,
                        ConfigurationManager.AppSettings[QuizChild.MeshIdentifierKey],
                        this.student.Name,
                        endpoint));

            this.responseClient = new QuizResponseClient(
                    this.site,
                    new NetPeerTcpBinding(QuizChild.PeerBindingKey),
                    address);
            this.responseClient.ClientCredentials.Peer.SetCertificate(
                StoreLocation.LocalMachine,
                StoreName.My,
                X509FindType.FindBySubjectDistinguishedName,
                QuizChild.CertificateSubjecDistinguishedName);
            this.responseClient.ClientCredentials.Peer.PeerAuthentication.CertificateValidationMode = X509CertificateValidationMode.PeerTrust;
            this.responseClient.ClientCredentials.Peer.PeerAuthentication.TrustedStoreLocation = StoreLocation.CurrentUser;
            
            responseClient.Open();

            //Announce student to teacher:
            this.Join();
        }


		
		private void Join()
		{
			Bitmap bitmap = new Bitmap(string.Concat(this.student.Name,".bmp"));
			MemoryStream stream = new MemoryStream();
			bitmap.Save(stream,ImageFormat.Bmp);
			stream.Flush();
			byte[] buffer = stream.GetBuffer();

			this.student.Image = buffer;
			this.managementClient.Join(student);
			this.student.Image = null;
		}

		public void SendResponse(QuizResponse response)
		{
			response.Responder = student;
			this.responseClient.SendResponse(response);
		}

		private void CloseChannel(IChannel channel)
		{
			if (channel.State == CommunicationState.Opened)
			{
				channel.Close();
			}
		}

		
		#region IQuizQuestion Members

		void IQuizQuestion.AddItem(QuizItem item)
		{
			this.form.AddQuizItem(item);
		}

		#endregion

		#region IQuizResponse Members

		void IQuizResponse.SendResponse(QuizResponse response)
		{
			this.form.DisplayGrade(response);
		}

		#endregion

		
		#region IDisposable Members

		void IDisposable.Dispose()
		{
			if (this.resolverService.State == CommunicationState.Opened)
			{
				this.resolverService.Close();
			}

			if (this.resolverService.State == CommunicationState.Opened)
			{
				this.resolverService.Close();
			}

			this.CloseChannel((IChannel)this.questionClient);
			this.CloseChannel((IChannel)this.managementClient);
			this.CloseChannel((IChannel)this.responseClient);
			
		}

		#endregion

		#region IQuizManagement Members

		void IQuizManagement.Join(Student student)
		{
		}

		void IQuizManagement.Announce(string announcement)
		{
			this.form.DisplayAnnouncement(announcement);
		}

		#endregion
    }

	
}
