using System;
using System.ServiceModel.Channels;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Net;
using System.Net.Sockets;
using System.ServiceModel;
using System.ServiceModel.Configuration;
using System.ServiceModel.PeerResolvers;

namespace WindowsCommunicationFoundationHandsOn.School
{
    public interface ICustomPeerResolverChannel : IPeerResolver, IClientChannel
    {
    }

	public class ResolverClient : PeerResolver
    {
        const string config = "CustomPeerResolverEndpoint";

        // Allow the clients to share referals
        public override bool CanShareReferrals
        {
            get { return true; }
        }

        // Register address for a node participating in a mesh identified by meshId with the resolver service
        public override object Register(string meshId, PeerNodeAddress nodeAddress, TimeSpan timeout)
        {
            using (ChannelFactory<ICustomPeerResolverChannel> factory = new ChannelFactory<ICustomPeerResolverChannel>(config))
            {
                using (ICustomPeerResolverChannel client = factory.CreateChannel())
                {
                    MaskScopeId(nodeAddress.IPAddresses);
                    int registrationId = client.Register(meshId, nodeAddress);
                    return registrationId;
                }
            }
        }

        // Unregister address for a node from the resolver service
        public override void Unregister(object registrationId, TimeSpan timeout)
        {
            using (ChannelFactory<ICustomPeerResolverChannel> factory = new ChannelFactory<ICustomPeerResolverChannel>(config))
            {
                using (ICustomPeerResolverChannel client = factory.CreateChannel())
                {
                    client.Unregister((int)registrationId);
                }
            }
        }

        // Updates a node's registration with the resolver service.
        public override void Update(object registrationId, PeerNodeAddress updatedNodeAddress, TimeSpan timeout)
        {
            using (ChannelFactory<ICustomPeerResolverChannel> factory = new ChannelFactory<ICustomPeerResolverChannel>(config))
            {
                using (ICustomPeerResolverChannel client = factory.CreateChannel())
                {
                    MaskScopeId(updatedNodeAddress.IPAddresses);
                    client.Update((int)registrationId, updatedNodeAddress);
                }
            }
        }

        // Query the resolver service for addresses associated with a mesh ID
        public override ReadOnlyCollection<PeerNodeAddress> Resolve(string meshId, int maxAddresses, TimeSpan timeout)
        {
            PeerNodeAddress[] addresses = null;

            using (ChannelFactory<ICustomPeerResolverChannel> factory = new ChannelFactory<ICustomPeerResolverChannel>(config))
            {
                using (ICustomPeerResolverChannel client = factory.CreateChannel())
                {
                    addresses = client.Resolve(meshId, maxAddresses);
                }
            }

            // If addresses couldn't be obtained, return empty collection
            if (addresses == null)
                addresses = new PeerNodeAddress[0];

            return new ReadOnlyCollection<PeerNodeAddress>(addresses);
        }

        // Since we send the IP addresses to the service, mask the scope ID (scopeIDs only have local significance)
        void MaskScopeId (ReadOnlyCollection<IPAddress> ipAddresses)
        {
            foreach (IPAddress address in ipAddresses)
            {
                if (address.AddressFamily == AddressFamily.InterNetworkV6)
                    address.ScopeId = 0;
            }
        }
    }
}

